define([
	"dojo/_base/lang",
	"dojo/_base/declare",
	"dojo/query",
	"dojo/dom-class",
	"dijit/registry",
	"dojo/json",
	"dojo/on",
	"dojo/promise/all",
	"dojox/mvc/at",
	"aps/_View",
	"aps/load",
	"dojo/text!./dapi.json",
	"dojo/text!./code.html",
	"./rules.js",
	"./sourceGenerator.js",
	"aps/Container",
	"aps/Panel",
	"aps/FieldSet",
	"aps/Tiles",
	"aps/Toolbar",
	"aps/Output",
	"aps/Button",
	"aps/CheckBox",
	"aps/Spinner",
	"aps/Select",
	"aps/TextBox"
], function(lang, declare, query, domClass, registry, JSON, on, all, at, _View, load, dapi, codeHtml, getRules, getSource, Container, Panel, FieldSet, Tiles, Toolbar, Output, Button, CheckBox, Spinner, Select, TextBox){
	dapi = JSON.parse(dapi, true);

	return declare(_View, {
		init: function(){
			var self = this,
				iframe,
				options,
				sourceOptions;

			function changeWidget(widgetName){
				var rules = getRules(widgetName),
					classDoc = dapi[widgetName],
					flags = new FieldSet(),
					fields = new FieldSet();

				options = lang.clone(rules.options);
				sourceOptions = lang.clone(rules.options);

				var containerSelector = registry.byId('containerSelector');
				containerSelector.set('options', rules.containers.map(function(containerName){
					return {value: containerName, label: containerName, selected: (containerName == 'aps/FieldSet')}
				}));
				containerSelector.set('disabled', false);

				var properties = registry.byId('properties');
				properties.removeAll();

				require([widgetName], function(Widget){
					classDoc.properties
						.filter(function(property){
							return !(property.private || (rules.properties[property.name] && rules.properties[property.name].hide));
						})
						.forEach(function(property){
							var widget = new Widget(lang.clone(rules.options)),
								field;
							var type = (rules.properties[property.name] && rules.properties[property.name].type) ?
								rules.properties[property.name].type : property.types[0];
							switch (type.toLowerCase()){
								case 'object':
									break;
								case 'boolean':
								case 'bool':
									field = new CheckBox({
										id: property.name,
										gridSize: 'md-2',
										description: property.name,
										optionalText: undefined,
										checked: widget.get(property.name)
									});
									options[property.name] = at('widget:' + property.name, 'checked');//.direction(at.from);
									on(field, 'click', function(){updateSource(field.get('id'), !field.get('checked'));});
									flags.addChild(field);
									break;
								case 'number':
									field = new Spinner({
										id: property.name,
										gridSize: 'md-3',
										label: property.name,
										minimum: 0,
										maximum: Infinity,
										optionalText: undefined,
										value: parseInt(widget.get(property.name))
									});
									options[property.name] = at('widget:' + property.name, 'value').direction(at.from);
									on(field.textbox, 'change', function(){updateSource(field.get('id'), field.get('value'));});
									fields.addChild(field);
									break;
								case 'select':
									field = new Select({
										id: property.name,
										gridSize: 'md-3',
										label: property.name,
										options: rules.properties[property.name].options,
										optionalText: undefined,
										value: widget.get(property.name)
									});
									options[property.name] = at('widget:' + property.name, 'value').direction(at.from);
									on(field, 'change', function(){updateSource(field.get('id'), field.get('value'));});
									fields.addChild(field);
									break;
								default :
									field = new TextBox({
										id: property.name,
										gridSize: 'md-3',
										label: property.name,
										optionalText: undefined,
										value: widget.get(property.name)
									});
									options[property.name] = at('widget:' + property.name, 'value').direction(at.from);
									on(field, 'change', function(){updateSource(field.get('id'), field.get('value'));});
									fields.addChild(field);
									break;
							}
						});

					properties.addChild(flags);
					properties.addChild(fields);
					properties.set('title', 'Properties for ' + widgetName);

					changeContainer(registry.byId('containerSelector').get('value'));
				});
			}

			function changeContainer(containerName){
				var widgetContainer = registry.byId('widgetContainer');
				widgetContainer.removeAll();

				var widgetName = registry.byId('widgetSelector').get('value'),
					rules = getRules(widgetName);

				require([widgetName], function(Widget){
					var widget = new Widget(lang.mixin(lang.clone(options), {id: widgetName}));
					rules.post(widget);

					switch (containerName){
						case 'aps/PageContainer':
							widgetContainer.addChild(widget);
							break;
						case 'aps/Panel':
							var panel = new Panel({title: widgetName + ' in ' + containerName});
							panel.addChild(widget);
							widgetContainer.addChild(panel);
							break;
						case 'aps/FieldSet':
							var panel = new Panel({title: widgetName + ' in ' + containerName});
							var fieldSet = FieldSet();
							fieldSet.addChild(widget);
							panel.addChild(fieldSet);
							widgetContainer.addChild(panel);
							break;
						case 'aps/Container':
							var panel = new Panel({title: widgetName + ' in ' + containerName});
							var innerContainer = new Container();
							innerContainer.addChild(widget);
							panel.addChild(innerContainer);
							widgetContainer.addChild(panel);
							break;
						case 'aps/Tiles':
							var tiles = new Tiles({title: widgetName + ' in ' + containerName});
							tiles.addChild(widget);
							widgetContainer.addChild(tiles);
							break;
						case 'aps/Toolbar':
							var panel = new Panel({title: widgetName + ' in ' + containerName});
							var toolbar = Toolbar();
							toolbar.addChild(widget);
							panel.addChild(toolbar);
							widgetContainer.addChild(panel);
							break;
					}
					showCode();
				});
			}

			function updateSource(property, value){
				sourceOptions[property] = value;
				showCode();
			}

			function showCode(){
				var widgetName = registry.byId('widgetSelector').get('value'),
					containerName = registry.byId('containerSelector').get('value'),
					sourceContainer = registry.byId('sourceContainer');

				if(iframe){
					sourceContainer.containerNode.removeChild(iframe);
				}
				iframe = document.createElement("iframe");
				iframe.setAttribute('frameborder', 0);
				iframe.width = '100%';
				sourceContainer.containerNode.appendChild(iframe);
				iframe.contentWindow.document.write((new String(codeHtml)).replace('${code}', getSource(containerName, widgetName, sourceOptions)));
				iframe.contentWindow.document.close();
				iframe.style.height = iframe.contentWindow.document.body.scrollHeight + 60 + "px";
			}

			all([
				load(['aps/Panel', {}, [
					['aps/FieldSet', {}, [
						['aps/Select', {
							id: 'widgetSelector',
							label: 'Widget',
							gridSize: 'md-5',
							options: Object.keys(dapi)
								.filter(function(className){
									return getRules(className) && dapi[className].mixins;
								})
								.sort()
								.map(function(className){
									return {value: className, label: className}
								})
						}],
						['aps/Select', {
							id: 'containerSelector',
							label: 'Container',
							gridSize: 'md-7',
							disabled: true
						}]
					]],
					['aps/FieldSet', {id: 'properties'}, []]
				]], self.domNode),
				load(['aps/Container', {title: 'Widget', id: 'widgetContainer'}, []], self.domNode),
				load(['aps/Container', {title: 'Source code', id: 'sourceContainer'}, []], self.domNode)
			]).then(function(){
				query('#widgetSelector').on('change', function(){
					changeWidget(this.value);
				});
				query('#containerSelector').on('change', function(){
					changeContainer(this.value);
				});
				changeWidget(registry.byId('widgetSelector').get('value'));
			});
		},
		onContext: function(){
			aps.apsc.hideLoading();
		}
	});
});