define([
	"dojo/_base/lang",
	"dojox/mvc/at",
	"dojox/mvc/StatefulArray",
	"aps/Memory",
	"aps/Status",
	"aps/Grid",
	"aps/Message",
	"aps/Tiles",
	"aps/Tile",
	"aps/Pie",
	"aps/Button",
	"aps/ToolbarButton",
	"aps/ToolbarSeparator",
	"aps/Output"
], function(lang, at, StatefulArray, Memory, Status, Grid, Message, Tiles, Tile, Pie, Button, ToolbarButton, ToolbarSeparator, Output){
	'use strict';

	var meta = {
		'defaultRules': {
			containers: ['aps/Panel', 'aps/FieldSet', 'aps/Container'],
			properties: {
				escapeHTML: {hide: true},
				'autoWidth': {hide: true},
				'templateString': {hide: true},
				'classesMap': {hide: true},
				'strength': {hide: true},
				'indeterminateHighContrastImage': {hide: true},
				'activeStep': {hide: true},
				'contextMenu': {hide: true},
				'onClick': {hide: true},
				'autoSize': {hide: true},
				'size': {hide: true},
				'store': {hide: true},
				'columns': {hide: true},
				'gridSize': {hide: true}
			},
			options: {
				placeHolder: 'Type value',
				label: 'Label of widget',
				description: 'This is description of field',
				hint: 'Hint about widget'
			},
			post: function(){
				//alert('POST!!!');
			}
		},
		'aps/Button': {
			properties: {
				type: {
					type: 'select',
					options: [
						{value: 'default', label: 'default'},
						{value: 'primary', label: 'primary'},
						{value: 'success', label: 'success'},
						{value: 'info', label: 'info'},
						{value: 'warning', label: 'warning'},
						{value: 'danger', label: 'danger'},
						{value: 'link', label: 'link'}]
				},
				iconClass: {
					type: 'select',
					options: [
						{value: 'fa-upload', label: 'fa-upload'},
						{value: 'fa-user', label: 'fa-user'},
						{value: 'fa-user-plus', label: 'fa-user-plus'},
						{value: 'fa-user-secret', label: 'fa-user-secret'},
						{value: 'fa-user-times', label: 'fa-user-times'},
						{value: 'fa-users', label: 'fa-users'}]
				},
				align: {
					type: 'select',
					options: [
						{value: 'left', label: 'left'},
						{value: 'right', label: 'right'}]
				},
				requireItems: {hide: true},
				requireSingleItem: {hide: true},
				useIcon: {hide: true},
				forceHideFor: {hide: true},
				iconName: {hide: true}
			},
			options: {
				iconClass: 'fa-user',
				align: 'left'
			}
		},
		'aps/CheckBox': {
			properties: {
				'value': {hide: true},
				'state': {hide: true}
			}
		},
		'aps/ComboBox': {
			properties: {
				'autoComplete': {hide: true}
			},
			options: {
				store: new Memory({
					idProperty: 'id',
					data: [{"name": "Alabama", "id": "AL"},
						{"name": "Alaska", "id": "AK"},
						{"name": "American Samoa", "id": "AS"},
						{"name": "Arizona", "id": "AZ"},
						{"name": "Arkansas", "id": "AR"},
						{"name": "Armed Forces Europe", "id": "AE"},
						{"name": "Armed Forces Pacific", "id": "AP"},
						{"name": "Armed Forces the Americas", "id": "AA"},
						{"name": "California", "id": "CA"},
						{"name": "Colorado", "id": "CO"},
						{"name": "Connecticut", "id": "CT"},
						{"name": "Delaware", "id": "DE"},
						{"name": "Alabama", "id": "AL2"},
						{"name": "Alaska", "id": "AK2"},
						{"name": "American Samoa", "id": "AS2"},
						{"name": "Arizona", "id": "AZ2"},
						{"name": "Arkansas", "id": "AR2"},
						{"name": "Armed Forces Europe", "id": "AE2"},
						{"name": "Armed Forces Pacific", "id": "AP2"},
						{"name": "Armed Forces the Americas", "id": "AA2"},
						{"name": "California", "id": "CA2"},
						{"name": "Colorado", "id": "CO2"},
						{"name": "Connecticut", "id": "CT2"},
						{"name": "Delaware", "id": "DE2"}]
				})
			}
		},
		'aps/DateTextBox': {
			properties: {
				dropDownPosition: {hide: true}
			}
		},
		'aps/Gauge': {
			properties: {
				percent: {hide: true},
				precision: {hide: true},
				width: {hide: true}
			},
			options: {
				legend: 'Legend about aps/Gauge'
			}
		},
		'aps/Grid': {
			containers: ['aps/PageContainer'],
			properties: {
				selectionMode: {
					type: 'select',
					options: [
						{value: 'single', label: 'single'},
						{value: 'multiple', label: 'multiple'},
						{value: '', label: 'null'}]
				},
				errorMessage: {hide: true},
				loadMessage: {hide: true},
				onRowClick: {hide: true},
				filters: {hide: true},
				saveState: {hide: true},
				apsResourceViewId: {hide: true}
			},
			options: {
				rowsPerPage: 5, //Todo: m.b. bug
				store: new Memory({
					idProperty: 'id',
					data: [{
						"id": 1,
						"first_name": "Dorothy",
						"last_name": "Stone",
						"email": "dstone0@virginia.edu",
						"country": "Ireland",
						"phone": "6-(888)715-7241",
						"disk_space": 82
					},
						{
							"id": 2,
							"first_name": "Sara",
							"last_name": "Gordon",
							"email": "sgordon1@nyu.edu",
							"country": "Brazil",
							"phone": "3-(367)831-9353",
							"disk_space": 33
						},
						{
							"id": 3,
							"first_name": "Beverly",
							"last_name": "Mason",
							"email": "bmason2@surveymonkey.com",
							"country": "Netherlands",
							"phone": "8-(253)084-4695",
							"disk_space": 58
						},
						{
							"id": 4,
							"first_name": "Jean",
							"last_name": "Hunter",
							"email": "jhunter3@merriam-webster.com",
							"country": "Brazil",
							"phone": "5-(634)157-2639",
							"disk_space": 87
						},
						{
							"id": 5,
							"first_name": "Justin",
							"last_name": "Butler",
							"email": "jbutler4@ning.com",
							"country": "Russia",
							"phone": "9-(417)354-7430",
							"disk_space": 40
						},
						{
							"id": 6,
							"first_name": "Harold",
							"last_name": "Campbell",
							"email": "hcampbell5@kickstarter.com",
							"country": "Netherlands",
							"phone": "1-(272)581-7876",
							"disk_space": 93
						},
						{
							"id": 7,
							"first_name": "Michael",
							"last_name": "Howard",
							"email": "mhoward6@exblog.jp",
							"country": "China",
							"phone": "3-(360)582-8422",
							"disk_space": 53
						},
						{
							"id": 8,
							"first_name": "Arthur",
							"last_name": "Ramirez",
							"email": "aramirez7@delicious.com",
							"country": "China",
							"phone": "6-(248)021-1806",
							"disk_space": 59
						},
						{
							"id": 9,
							"first_name": "Kevin",
							"last_name": "Cole",
							"email": "kcole8@netscape.com",
							"country": "Montenegro",
							"phone": "0-(481)188-5737",
							"disk_space": 51
						},
						{
							"id": 10,
							"first_name": "Betty",
							"last_name": "Powell",
							"email": "bpowell9@nationalgeographic.com",
							"country": "Portugal",
							"phone": "7-(699)660-3003",
							"disk_space": 23
						}]
				}),
				columns: [
					{name: 'User ID', field: 'id'},
					{
						name: 'Full name',
						field: 'first_name',
						filter: {title: 'Full name'},
						renderCell: function(row, data){
							return row['first_name'] + ' ' + row['last_name'];
						}
					}, {
						name: 'E-Mail', field: 'email',
						renderCell: function(row, data){
							return new Output({content: '<a href="mailto:' + data + '">' + data + '</a>'});
						}
					},
					{name: 'Country', field: 'country'},
					{name: 'Phone', field: 'phone'},
					{
						name: 'Disk space', field: 'disk_space', align: 'right', escapeHTML: false,
						renderCell: function(row, data){
							return data + ' <strong>GB</strong><br>' + (data * 1000 * 1000 / 1024).toFixed(2) + ' <strong>MB</strong><br>';
						}
					}, {
						name: 'Action',
						renderCell: function(row, data){
							return new Button({
								timeout: 200,
								title: 'Activate'
							});
						}
					}
				]
			}
		},
		'aps/ListSwitcher': {
			containers: ['aps/PageContainer'],
			post: function(listSwitcher){
				listSwitcher.set('store', new Memory({
					idProperty: 'id',
					data: [{
						"id": 1,
						"first_name": "Kenneth",
						"last_name": "Patterson",
						"emails_limit": 128,
						"disk_limit": 290,
						"emails_used": 64,
						"disk_used": 140
					}, {
						"id": 2,
						"first_name": "Irene",
						"last_name": "Harvey",
						"emails_limit": 149,
						"disk_limit": 615,
						"emails_used": 45,
						"disk_used": 144
					}, {
						"id": 3,
						"first_name": "Ernest",
						"last_name": "Carroll",
						"emails_limit": 129,
						"disk_limit": 576,
						"emails_used": 17,
						"disk_used": 381
					}, {
						"id": 4,
						"first_name": "Tina",
						"last_name": "Ramirez",
						"emails_limit": 84,
						"disk_limit": 160,
						"emails_used": 64,
						"disk_used": 147
					}, {
						"id": 5,
						"first_name": "Emily",
						"last_name": "Vasquez",
						"emails_limit": 154,
						"disk_limit": 320,
						"emails_used": 50,
						"disk_used": 200
					}, {
						"id": 6,
						"first_name": "Willie",
						"last_name": "Ferguson",
						"emails_limit": 119,
						"disk_limit": 293,
						"emails_used": 8,
						"disk_used": 112
					}, {
						"id": 7,
						"first_name": "Paul",
						"last_name": "Anderson",
						"emails_limit": 156,
						"disk_limit": 51,
						"emails_used": 54,
						"disk_used": 2
					}]
				}));
				var tiles = new Tiles();
				var tile = new Tile({
					title: at('rel:last_name'),
					contextMenu: [
						{icon: "fa-cog", label: "View Settings"},
						{icon: "fa-minus-circle", label: "Disable Widget"}
					],
					leftBtn: {title: "Add New user", iconClass: "fa-plus"},
					onClick: function(){
						console.log('click');
					}
				});

				tile.addChild(new Pie({
					maximum: at('rel:', 'emails_limit'),
					value: at('rel:', 'emails_used'),
					legend: 'Used ${value} of ${maximum} Boxes'
				}));

				tiles.addChild(tile);
				listSwitcher.addChild(tiles);

				listSwitcher.addChild(new Grid({
					columns: [
						{name: 'User ID', field: 'id'},
						{name: 'First name', field: 'first_name'},
						{name: 'Last name', field: 'last_name'},
						{name: 'E-Mail limit', field: 'emails_limit', align: 'right'},
						{name: 'E-Mail used', field: 'emails_used', align: 'right'}
					]
				}));
			}
		},
		'aps/Message': {
			properties: {
				type: {
					type: 'select',
					options: [
						{value: 'success', label: 'success'},
						{value: 'info', label: 'info'},
						{value: 'warning', label: 'warning'},
						{value: 'danger', label: 'danger'},
						{value: 'link', label: 'link'}]
				}
			},
			options: {
				type: 'success'
			}
		},
		'aps/MessageList': {
			containers: ['aps/PageContainer'],
			properties: {cols: {hide: true}},
			options: {
				type: 'success',
				visible: true,
				title: 'Important messages'
			},
			post: function(messageList){
				dijit.registry.byId('apsPageContainer').set('messageList', messageList);

				var types = ['error', 'update', 'limits', 'progress'];
				for (var i = 0; i < 15;){
					var type = types[i++ % 4];
					messageList.addChild(new Message({
						description: '# ' + i + ' Message type ' + type,
						type: type
					}));
				}
			}
		},
		'aps/MultiSelect': {
			properties: {
				values: {hide: true}
			},
			options: {
				titleLeft: "Available:",
				store: new Memory({
					idProperty: "value",
					data: [{
						label: "SiteBuilder",
						value: 1
					}, {
						label: "2 Additional Gigabytes",
						value: 2
					}, {
						label: "5 Additional Gigabytes",
						value: 3
					}]
				}),
				titleRight: "Selected:",
				values: new StatefulArray([1]),
				labelAttr: "label"
			}
		},
		'aps/Output': {
			options: {
				value: 'Widget’s content may be specified through the value property or through the content property.'
			}
		},
		'aps/Panel': {
			containers: ['aps/PageContainer'],
			properties: {
				cols: {hide: true},
				label: {hide: true},
				optionalText: {hide: true}
			},
			options: {
				title: 'Title of panel',
				description: 'Description about content of panel',
				hint: 'Hint of panel'
			}
		},
		'aps/Password': {
			properties: {
				confirmation: {hide: true}
			},
			options: {
				value: 'veryVery$trongPa$$W0rd!'
			}
		},
		'aps/Select': {
			properties: {
				state: {hide: true}
			},
			options: {
				options: [
					{value: "TN", label: "Tennessee"},
					{value: "VA", label: "Virginia"},
					{value: "WA", label: "Washington"},
					{value: "FL", label: "Florida"},
					{value: "CA", label: "California"}
				]
			}
		},
		'aps/Spinner': {
			properties: {
				state: {hide: true}
			},
			options: {
				maximum: 100,
				minimum: 0
			}
		},
		'aps/Status': {
			properties: {
				status: {
					type: 'select',
					options: [
						{value: 'disabled', label: 'disabled'},
						{value: 'inProgress', label: 'inProgress'},
						{value: 'ready', label: 'ready'},
						{value: 'warn', label: 'warn'},
						{value: 'error', label: 'error'}]
				}
			}
		},
		'aps/Switch': {
			properties: {
				status: {
					type: 'select',
					options: [
						{value: 'on', label: 'on'},
						{value: 'off', label: 'off'},
						{value: 'disabling', label: 'disabling'},
						{value: 'offByError', label: 'offByError'},
						{value: 'enabling', label: 'enabling'}]
				}
			},
			options: {
				onClick: function(){
					var self = this;
					switch (this.status){
						case "on":
							this.set("status", "disabling");
							setTimeout(function(){
								self.set("status", "off");
							}, 50);
							break;
						case "disabling":
							break;
						case "off":
							this.set("status", "enabling");
							setTimeout(function(){
								self.set("status", "on");
							}, 50);
							break;
						case "offByError":
							this.set("status", "enabling");
							setTimeout(function(){
								self.set("status", "offByError");
							}, 50);
							break;
						case "enabling":
							break;
					}
				}
			}
		},
		'aps/TextArea': {
			properties: {
				cols: {hide: true},
				tooltipPosition: {hide: true},
				state: {hide: true}
			},
			options: {
				rows: 4
			}
		},
		'aps/TextBox': {
			options: {
				promptMessage: 'This is prompt message!'
			}
		},
		'aps/Tile': {
			containers: ['aps/Tiles'],
			properties: {
				info: {hide: true},
				iconRightCornerClass: {
					type: 'select',
					options: [
						{value: 'fa-upload', label: 'fa-upload'},
						{value: 'fa-user', label: 'fa-user'},
						{value: 'fa-user-plus', label: 'fa-user-plus'},
						{value: 'fa-user-secret', label: 'fa-user-secret'},
						{value: 'fa-user-times', label: 'fa-user-times'},
						{value: 'fa-users', label: 'fa-users'}]
				},
				type: {hide: true}
			},
			options: {
				iconRightCornerClass: 'fa-users',
				info: new Status(), //todo: bug
				onClick: function(){}
			},
			post: function(tile){
				tile.set('info', new Status()); //todo: bug
				tile.addChild(new Output({
					value: 'aps/Output as content of Tile'
				}));
			}
		},
		'aps/Tiles': {
			containers: ['aps/PageContainer'],
			properties: {
				selectionMode: {
					type: 'select',
					options: [
						{value: 'single', label: 'single'},
						{value: 'multiple', label: 'multiple'},
						{value: '', label: 'null'}
					]
				},
				type: {
					type: 'select',
					options: [
						{value: 'notify', label: 'notify'},
						{value: 'widgets', label: 'widgets'}
					]
				}
			},
			options: {
				title: 'Resource usage by users'
			},
			post: function(tiles){
				tiles.set('store', new Memory({
					idProperty: 'id',
					data: [{
						"id": 1,
						"first_name": "Kenneth",
						"last_name": "Patterson",
						"emails_limit": 128,
						"disk_limit": 290,
						"emails_used": 64,
						"disk_used": 140
					}, {
						"id": 2,
						"first_name": "Irene",
						"last_name": "Harvey",
						"emails_limit": 149,
						"disk_limit": 615,
						"emails_used": 45,
						"disk_used": 144
					}, {
						"id": 3,
						"first_name": "Ernest",
						"last_name": "Carroll",
						"emails_limit": 129,
						"disk_limit": 576,
						"emails_used": 17,
						"disk_used": 381
					}, {
						"id": 4,
						"first_name": "Tina",
						"last_name": "Ramirez",
						"emails_limit": 84,
						"disk_limit": 160,
						"emails_used": 64,
						"disk_used": 147
					}, {
						"id": 5,
						"first_name": "Emily",
						"last_name": "Vasquez",
						"emails_limit": 154,
						"disk_limit": 320,
						"emails_used": 50,
						"disk_used": 200
					}, {
						"id": 6,
						"first_name": "Willie",
						"last_name": "Ferguson",
						"emails_limit": 119,
						"disk_limit": 293,
						"emails_used": 8,
						"disk_used": 112
					}, {
						"id": 7,
						"first_name": "Paul",
						"last_name": "Anderson",
						"emails_limit": 156,
						"disk_limit": 51,
						"emails_used": 54,
						"disk_used": 2
					}]
				}));
				var tile = new Tile({
					iconRightCornerClass: 'fa-user',
					title: at('rel:last_name'),
					contextMenu: [
						{icon: "fa-cog", label: "View Settings"},
						{icon: "fa-minus-circle", label: "Disable Widget"}
					],
					leftBtn: {title: "Add New user", iconClass: "fa-plus"},
					onClick: function(){}
				});

				tile.addChild(new Pie({
					maximum: at('rel:', 'emails_limit'),
					value: at('rel:', 'emails_used'),
					legend: 'Used ${value} of ${maximum} Boxes'
				}));
				tiles.addChild(tile);
			}
		},
		'aps/Toolbar': {
			containers: ['aps/Panel', 'aps/Container'],
			post: function(toolbar){
				toolbar.addChild(new ToolbarButton({
					label: 'Button 1'
				}));
				toolbar.addChild(new ToolbarButton({
					label: 'Button 2'
				}));
				toolbar.addChild(new ToolbarButton({
					label: 'Button 3'
				}));
				toolbar.addChild(new ToolbarSeparator());
				toolbar.addChild(new ToolbarButton({
					label: 'Button 4'
				}));
				toolbar.addChild(new ToolbarButton({
					label: 'Button 5'
				}));
				toolbar.addChild(new ToolbarButton({
					label: 'Button 6'
				}));
			}
		},
		'aps/ToolbarButton': {
			containers: ['aps/Toolbar'],
			properties: {
				type: {
					type: 'select',
					options: [
						{value: 'default', label: 'default'},
						{value: 'primary', label: 'primary'},
						{value: 'success', label: 'success'},
						{value: 'info', label: 'info'},
						{value: 'warning', label: 'warning'},
						{value: 'danger', label: 'danger'},
						{value: 'link', label: 'link'}]
				},
				iconClass: {
					type: 'select',
					options: [
						{value: 'fa-upload', label: 'fa-upload'},
						{value: 'fa-user', label: 'fa-user'},
						{value: 'fa-user-plus', label: 'fa-user-plus'},
						{value: 'fa-user-secret', label: 'fa-user-secret'},
						{value: 'fa-user-times', label: 'fa-user-times'},
						{value: 'fa-users', label: 'fa-users'}]
				},
				align: {
					type: 'select',
					options: [
						{value: 'left', label: 'left'},
						{value: 'right', label: 'right'}]
				},
				requireItems: {hide: true},
				requireSingleItem: {hide: true},
				useIcon: {hide: true},
				forceHideFor: {hide: true},
				iconName: {hide: true}
			},
			options: {
				iconClass: 'fa-user',
				align: 'left'
			}
		},
		'aps/bill/CreditCard': {}, //Todo: sad
		'aps/tiles/PieTile': {
			containers: ['aps/Tiles'],
			properties: {
				info: {hide: true},
				iconRightCornerClass: {
					type: 'select',
					options: [
						{value: 'fa-upload', label: 'fa-upload'},
						{value: 'fa-user', label: 'fa-user'},
						{value: 'fa-user-plus', label: 'fa-user-plus'},
						{value: 'fa-user-secret', label: 'fa-user-secret'},
						{value: 'fa-user-times', label: 'fa-user-times'},
						{value: 'fa-users', label: 'fa-users'}]
				},
				type: {hide: true},
				percent: {hide: true}
			},
			options: {
				iconRightCornerClass: 'fa-users',
				info: new Status(),
				value: 30,
				onClick: function(){}
			}
		},
		'aps/tiles/ResourceUsage': {
			containers: ['aps/Tiles'],
			properties: {
				info: {hide: true},
				iconRightCornerClass: {
					type: 'select',
					options: [
						{value: 'fa-upload', label: 'fa-upload'},
						{value: 'fa-user', label: 'fa-user'},
						{value: 'fa-user-plus', label: 'fa-user-plus'},
						{value: 'fa-user-secret', label: 'fa-user-secret'},
						{value: 'fa-user-times', label: 'fa-user-times'},
						{value: 'fa-users', label: 'fa-users'}]
				},
				type: {hide: true},
				percent: {hide: true}
			},
			options: {
				iconRightCornerClass: 'fa-users',
				info: new Status(),
				limit: 500,
				threshold: 450,
				resourceName: 'GB',
				value: 466,
				description: 'Used!',
				onClick: function(){}
			}
		},
		'aps/tiles/UsageInfoTile': {
			containers: ['aps/Tiles']
		}
	};


	return function(className){
		var ignoredClasses = [
			'aps/ActiveItem',
			'aps/ActiveList',
			'aps/InfoBoard',
			'aps/ResourceStore',
			'aps/Slider',
			'aps/ToolsItem',
			'aps/ToolsList',
			'aps/WizardControl',
			'aps/Container',
			'aps/FieldSet',
			'aps/ToolbarSeparator',
			'aps/PageContainer'
		];
		if(ignoredClasses.indexOf(className) > -1 || !className.match(/^aps\/[A-Za-z]/)) return false;

		var rules = meta[className];
		var defaultRules = lang.clone(meta['defaultRules']);
		if(rules === undefined) return defaultRules;

		rules.options = lang.mixin(defaultRules.options, rules.options || {});
		rules.properties = lang.mixin(defaultRules.properties, rules.properties || {});
		return lang.mixin(defaultRules, rules);
	};
});

